import logging
from . import x_data_cleaner
from . import open_ai

logging.basicConfig(level=logging.INFO)

def main_big_five(data_path, openai_key):
    print(data_path)
    logging.info("Starting main_big_five function.")
    logging.info("Getting prompt content.")
    prompt_content = get_prompt_content(data_path)
    logging.info("Generating prompt.")
    prompt = get_prompt(prompt_content)
    logging.info("Creating messages.")
    messages = get_messages(prompt)
    logging.info("Getting result from LLM.")
    result = get_result(messages, openai_key)
    logging.info("Extracting result.")
    extracted_result = extract_result(result, openai_key)    
    logging.info("Finished main_big_five function.")
    return extracted_result

def get_prompt(text):
    return f"""
    Analyze the person-generated text from sentence level, determine the person’s levels of Openness, Conscientiousness, Extraversion, Agreeableness, and Neuroticism. Only return Low or High.

    Text:
    {text}

    Level: Let’s think step by step:
    """

def get_messages(prompt):
    """
    Creates the messages that are then used for the request to OpenAI.
    """
    messages=[
        {"role": "system", "content": "You are a psychologist who is given some tweets, and based on those tweets, you make a Big Five personality analysis."},
        {"role": "user", "content": prompt}
    ]
    return messages

def get_prompt_content(data_path):
    """
    Returns content that can be inserted into the prompt.
    """
    get_prompt_content = x_data_cleaner.main_x_data_cleaner(data_path)
    return get_prompt_content

def get_result(prompt, openai_key):
    """
    Sends a request to OpenAI and receives the result of the personality analysis.
    """
    result = open_ai.main_openai(prompt, openai_key)
    return result

def extract_result(result, openai_key):
    """
    The function extracts the result and returns a dictionary.
    """
    prompt ="""You are receiving a text as input that contains a personality analysis of an individual. The analysis is based on the Big Five personality traits, which include openness, conscientiousness, extraversion, agreeableness, and neuroticism. Please extract the analysis results in the following format: {"openness": "low", "conscientiousness": "high", "extraversion": "low", "agreeableness": "high", "neuroticism": "low"}."""
    messages = [{"role": "user", "content": prompt}, {"role": "user", "content": result}]
    result = open_ai.main_openai(messages)
    return result



from openai import OpenAI

def main_openai(messages, openai_key, model="gpt-4", temperature=0.0):
    """
    Executes requests to OpenAI.
    """
    client = OpenAI(api_key = openai_key)
    completion = client.chat.completions.create(
        model=model,
        messages=messages,
        temperature=temperature
    )
    return completion.choices[0].message.content




from setuptools import setup, find_packages

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name='persai',
    version='0.0.3',
    packages=find_packages(),
    install_requires=required,
    description="Big Five Personality Analysis based on Twitter (X) posts.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Max Hager',
    author_email='maxhager28@gmail.com',
    url='https://github.com/yachty66/persai',
    include_package_data=True
)



import json

def main_x_data_cleaner(path_to_data):
    """
    The main function returns 20 posts in a formatted prompt.
    """
    print("Starting main_x_data_cleaner function.")
    valid_json_objects = load_json_objects(path_to_data)
    print("Loaded valid JSON objects.")
    filtered_json_objects = filter_entities(valid_json_objects)
    print("Filtered JSON objects.")
    full_text_objects = get_full_text_objects(filtered_json_objects)
    print("Got full text objects.")
    full_prompt_content = generate_full_prompt(full_text_objects)
    print("Generated full prompt content.")
    print("Finished main_x_data_cleaner function.")
    return full_prompt_content

# load all json objects which are not broken into a list
def load_json_objects(filename):
    """
    Takes the path to the JavaScript file containing the Twitter data and returns a list of JSON objects.
    """
    valid_objects = []
    decoder = json.JSONDecoder()
    with open(filename, 'r') as f:
        text = f.read()
        pos = 0
        while pos < len(text):
            try:
                obj, pos = decoder.raw_decode(text, pos)
                valid_objects.append(obj)
            except json.JSONDecodeError:
                # Find the next position where { starts
                next_pos = text.find('{', pos)
                if next_pos == -1:
                    break
                pos = next_pos
    return valid_objects



def filter_entities(json_objects):
    """
    Get all posts with no entities, i.e. only posts that are purely from users.
    """
    return [obj for obj in json_objects if all(not obj['tweet']['entities'][key] for key in obj['tweet']['entities'])]

def get_full_text_objects(filtered_json_objects):
    """
    gets the full text of the tweets
    """
    # Assuming `data` is your dictionary
    full_text = []
    for i in range(len(filtered_json_objects)):
        full_text.append(filtered_json_objects[i]['tweet']['full_text'])
    return full_text

def generate_full_prompt(full_text_objects):
    """
    Generates a full prompt with which a request can be sent to OpenAI.
    """
    big_string = ""
    for i in range(20):
        big_string += "---\n" + full_text_objects[i] + "\n---\n"
    return big_string